{% extends "oauthx/base.html.j2" %}
{% set page_title=_("Page not found") %}

{% block content %}
<section class="action">
  <div class="text">
    <p>{{ _("The page you requested does not exist.") }}</p>
  </div>
</section>
{% endblock %}