# Copyright (C) 2023 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
import fastapi

from oauthx.server.handlers import BaseLoginRequestHandler
from oauthx.server.request import Request


class LoginRequestHandler(BaseLoginRequestHandler):
    __module__: str = 'oauthx.server.ref'

    async def handle(self, request: Request) -> fastapi.Response:
        if request.subject is None:
            return fastapi.Response(
                status_code=401,
                headers={'WWW-Authenticate': 'Basic'}
            )
        
        raise NotImplementedError