{% extends "oauthx/base.html.j2" %}
{% set page_title="Hello world!" %}


{% block content %}
<section class="action">
  <div class="text">
    <p>To implement authentication you must provide a concrete class of <tt>BaseLoginRequestHandler</tt>.</p>
  </div>
</section>
{% endblock %}