# Copyright (C) 2023 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
from typing import Literal

import pydantic

from oauthx.lib.types import ScopeType
from .basegrant import BaseGrant


class ClientCredentialsGrant(BaseGrant):
    grant_type: Literal['client_credentials'] = pydantic.Field(
        default=...,
        description="This value **must** be `client_credentials`."
    )

    scope: ScopeType = pydantic.Field(
        default_factory=ScopeType,
        description=(
            "The space-delimited scope of the access request. The client "
            "**must** allow the given scope."
        )
    )