# Copyright (C) 2023 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
from typing import Literal

import pydantic

from .baseresponsemode import BaseResponseMode


class FragmentResponseMode(BaseResponseMode):
    response_mode: Literal['fragment', 'fragment.jwt']
    response_type: Literal['id_token', 'code token', 'id_token token', 'code id_token token']

    @pydantic.field_validator('response_mode', mode='before')
    def preprocess_response_mode(cls, value: str | None) -> str:
        if value is None:
            value = 'fragment'
        return value
