# Copyright (C) 2023 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
import fastapi

from oauthx.resource import ResourceRoute
from oauthx.server.config import Config
from oauthx.server.request import Request


class UserInfoRoute(ResourceRoute):
    config: Config

    def setup_request(self, request: fastapi.Request) -> Request:
        request = Request(request.scope, request.receive)
        return request