{% extends "oauthx/base.html.j2" %}
{% set page_title=_("Confirm your Telegram account") %}


{% block content %}
<section class="action">
  <div class="text strong-primary">
    {% markdown %}{% trans client_name=client_name %}
    You must verify your Telegram account to continue to **{{ client_name }}**.
    {% endtrans %}{% endmarkdown %}
    <p>{% trans %}Click or tap the button below to login using Telegram and confirm your account.{% endtrans %}
    </br>
    </br>
    <p style="display:flex;justify-content:center;">
      <script async
        src="https://telegram.org/js/telegram-widget.js?22"
        data-telegram-login="{{ telegram_bot_name }}"
        data-size="large"
        data-auth-url="{{ telegram_callback_url }}"
        data-request-access="write"
      >
      </script>
    </p>
  </div>
</section>
{% endblock %}